/*
* This file is part of fwmon
* Copyright (c) 2001,2002 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char rcsid[]=
	"@(#) $Id: util.c,v 1.10 2002/01/31 09:32:34 gianni Exp $";

#include "g_globals.h"
#include "util.h"
#include <ctype.h>

char errstr[64];

/* ICMP type names */
icmp_typename_t icmp_tnames[]={
	{0, "ICMP_ECHO_REPLY"},
	{3, "ICMP_DEST_UNCREACH"},
	{4, "ICMP_SOURCE_QUENCH"},
	{5, "ICMP_REDIRECT"},
	{8, "ICMP_ECHO_REQUEST"},
	{9, "ICMP_ROUTER_ADVERTISEMENT"},
	{10, "ICMP_ROUTER_SOLICITATION"},
	{11, "ICMP_TIME_EXCEED"},
	{12, "ICMP_BAD_PARAM"},
	{13, "ICMP_TIMESTAMP_REQUEST"},
	{14, "ICMP_TIMESTAMP_REPLY"},
	{15, "ICMP_INFO_REQUEST"},
	{16, "ICMP_INFO_REPLY"},
	{17, "ICMP_MASK_REQUEST"},
	{18, "ICMP_MASK_REPLY"},
	{0, NULL},
};

/* ICMP type/code names */
icmp_name_t icmp_names[]={
	{3, 0,  "ICMP_NET_UNREACH"},
	{3, 1,  "ICMP_HOST_UNREACH"},
	{3, 2,  "ICMP_PROTO_UNREACH"},
	{3, 3,  "ICMP_PORT_UNCREACH"},
	{3, 4,  "ICMP_FRAG_NEEDED_BUT_DF_SET"},
	{3, 5,  "ICMP_SOURCE_ROUTE_FAILED"},
	{3, 6,  "ICMP_NET_UNKNOWN"},
	{3, 7,  "ICMP_HOST_UNKNOWN"},
	{3, 8,  "ICMP_SOURCE_HOST_ISOLATED"},
	{3, 9,  "ICMP_NET_ADMIN_PROHIBITED"},
	{3, 10, "ICMP_HOST_ADMIN_PROHIBITED"},
	{3, 11, "ICMP_NET_UNREACH_TOS"},
	{3, 12, "ICMP_HOST_UNREACH_TOS"},
	{3, 13, "ICMP_COMMS_FILTERED"},
	{3, 14, "ICMP_PRECEDENCE_VIOLATION"},
	{3, 15, "ICMP_PRECEDENCE_CUTOFF"},
	{5, 0,  "ICMP_REDIRECT_NET"},
	{5, 1,  "ICMP_REDIRECT_HOST"},
	{5, 2,  "ICMP_REDIRECT_TOS_NET"},
	{5, 3,  "ICMP_REDIRECT_TOS_HOST"},
	{11, 0, "ICMP_TTL_EXCEEDED"},
	{11, 1, "ICMP_TTL_EXCEEDED_IN_REASSEMBLY"},
	{12, 0, "ICMP_IP_HEADER_BAD"},
	{12, 1, "ICMP_IP_OPTION_MISSING"},
	{0, 0, NULL},
};

char *strtoupper(char *in)
{
	char *tmp=in;
	
	/* AUDIT: As long as get_proto() is the only caller
	 * of this function, bounds checking is not needed, as
	 * get_proto() gives us data straight from getprotobynumber()
	 */
	do {
		*tmp = toupper(*tmp);
	}while ( *(tmp++) != '\0' );
	
	return in;
}

const char *get_tcpflags(struct tcphdr *tcp)
{
	static char buf[16];
	char *b=(char *)&buf;

	memset(b, 0, 16);
	
	strncat(b, "[", 1);
	if ( tcp->ack ) strncat(b, "A", 1);
	if ( tcp->psh ) strncat(b, "P", 1);
	if ( tcp->rst ) strncat(b, "R", 1);
	if ( tcp->syn ) strncat(b, "S", 1);
	if ( tcp->fin ) strncat(b, "F", 1);
	if ( tcp->urg ) strncat(b, "U", 1);
	strncat(b, "]", 1);

	return b;
}

const char *get_proto(int p)
{
	struct protoent *proto;

	switch (p) {
		case 1: return "ICMP";
		case 2: return "IGMP";
		case 6: return "TCP";
		case 17: return "UDP";
		case 41: return "IPv6";
		case 43: return "IPv6-route";
		case 44: return "IPv6-frag";
		case 50: return "IPv6-crypt";
		case 51: return "IPv6-auth";
		case 58: return "IPv6-icmp";
		case 59: return "IPv6-nonxt";
		case 60: return "IPv6-opts";
		case 94: return "IPIP";
		case 98: return "ENCAP";
	}
	
	if ( flag(FLAG_CHROOT) ) {
		sprintf(errstr, "(%i)", p);
		return errstr;
	}
	
	proto = getprotobynumber(p);

	/* AUDIT: Callers no longer need check for NULL */
	if (proto)
		return strtoupper(proto->p_name);
	else
		sprintf(errstr, "(%i)", p);

	return errstr;
}

int format_ip ( u_long ip, char *txt )
{
	struct in_addr addr;
	addr.s_addr=ip;
	strncpy( txt, inet_ntoa(addr), 31 );
	return 1;
}

/* this is disgusting */
const char *icmp_type(u_char type, u_char code)
{
	int i;

	for (i=0; icmp_names[i].name; i++)
	{
		if ( type==icmp_names[i].type && code==icmp_names[i].code ) {
			return icmp_names[i].name;
		}
	}

	for (i=0; icmp_tnames[i].name; i++)
       	{
		if ( type == icmp_tnames[i].type ) {
			return icmp_tnames[i].name;
		}
	}
	
	return "ICMP_UNKNOWN";
}
